/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved

	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.

	This file is under Perforce control
	$Id: //depot/express/fcs992/include/avs/arr.h#1 $
*/

#ifndef ARR_INCLUDED
#define ARR_INCLUDED

#include <avs/port.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _ARRtype {
   char name[64];
   char *(*palloc) PTARGS((int,char *));/* Takes length and data ptr */
   char *(*prealloc) PTARGS((char *,int,char *));
   void (*pfree) PTARGS((char *,char *));/* Free's a previously allocated ptr */
   /* Reads an array from the connection */
   int (*pread) PTARGS((void *,int *,char **, int *, int));
   /* Writes an array to the connection */
   int (*pwrite) PTARGS((void *,int,char *,int, int)); 
   unsigned int code;
   struct _ARRtype *next;
} ARRtype;

typedef struct _ARRarray {
   ARRtype *arr_type;
   int	   d_type;
   char	   *d_ptr;
   int	   d_len;
   unsigned int     refcnt:28;
   unsigned int     trace:1;
   char    *info;
   int (*update) PTARGS((void *, char *, int, int, int));
   void    *upd_arg;
   struct _ARRarray *next;
} ARRarray;

/* Flags to the ARRupdate routine */
#define ARR_NOT_MODIFIED  2
#define ARR_LAST_UPDATE	  1

void	ARRinit PTARGS((void));
int	ARRset_trace PTARGS((void *, int));
int	ARRget_trace PTARGS((void *));
void	ARRset_default_type PTARGS((ARRtype *));
void    ARRset_comm_funcs();
ARRtype *ARRnew_type PTARGS((char *,char *(*)(int,char *), 
			     char *(*)(char *,int,char *), 
			     void (*)(char *, char *),
			     int (*)(void *,int *,char **,int *,int), 
			     int (*)(void *,int,char *,int,int)));
ARRtype *ARRname_to_type PTARGS((char *));
ARRtype *ARRcode_to_type PTARGS((unsigned int));

void *	ARRalloc PTARGS((ARRtype *, int, int, char *));
void *	ARRalloc_f PTARGS((ARRtype *, int, int, char *, char *, int));
void *	ARRrealloc PTARGS((void *, int, int, char *));
void *	ARRrealloc_f PTARGS((void *, int, int, char *, char *, int));
void	ARRincr_refcnt PTARGS((void *)); 
void	ARRincr_refcnt_f PTARGS((void *, char *, int));
void 	ARRfree PTARGS((void *));
void 	ARRfree_f PTARGS((void *, char *, int));
void	ARRfree_no_update_f PTARGS((void *, char *, int));
void    ARRupdate PTARGS((void *, int final));
int	ARRnew_managed PTARGS((ARRtype *, void *, int, int, char *));
int	ARRnew_unmanaged PTARGS((void *, int, int, char *));
int	ARRvalid PTARGS((void *));
int	ARRarray_read PTARGS((void *conn, int *typep, char **pptr, int *lenp, int mode));
int	ARRarray_write PTARGS((void *conn, int type, char *ptr, int len, int mode));
int	ARRrefcnt PTARGS((void *));
void	ARRexit PTARGS((void));

int  ARRhas_update_func PTARGS((void *ptr));
void ARRset_update_func PTARGS((void *ptr, 
				int (*func) (void *, char *, int, int, int),
				void *upd_arg));

#define ARR_ARRAY_RD	0
#define ARR_ARRAY_WR	1
#define ARR_ARRAY_RW	2

/* These macros enable the array routines to report from where
 * they are being called.
 */

#define ARRalloc(arr_type,d_type,d_len,arr_info) ARRalloc_f((arr_type), \
							    (d_type), \
							    (d_len), \
							    (arr_info), \
							    __FILE__, __LINE__)
#define ARRrealloc(ptr,d_type,d_len,arr_info) ARRrealloc_f((ptr), \
							   (d_type), \
							   (d_len), \
							   (arr_info), \
							   __FILE__, __LINE__)
#define ARRfree(ptr) ARRfree_f((ptr), __FILE__, __LINE__)
#define ARRfree_no_update(ptr) \
	ARRfree_no_update_f((ptr), __FILE__, __LINE__)
#define ARRincr_refcnt(ptr) ARRincr_refcnt_f(((char*)ptr), __FILE__, __LINE__)

#ifdef __cplusplus
} /* End of extern "C" */
#endif

#endif
